/* ctype.h,v 1.14 2004/09/04 06:22:16 bird Exp */
/** @file
 *
 * InnoTek LIBC - Character type querying.
 *
 * Copyright (c) 2004 knut st. osmundsen <bird@innotek.de>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with This program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _CTYPE_H_
#define	_CTYPE_H_

#include <sys/cdefs.h>

#if !defined(__InnoTekLIBC_locale_h__)
__BEGIN_DECLS
/**
 * Simplified Ctype structure for inline functions.
 */
extern struct
{
    /** All uppercased characters. */
    unsigned char   auchUpper[256];
    /** All lowercased characters. */
    unsigned char   auchLower[256];
    /** Bit flags for every character (for isXXX() function series). */
    unsigned short  ausfType[256];
} __libc_GLocaleCtype;

/**
 * Simplified Default Ctype structure for inline functions.
 */
extern const struct
{
    /** Bit flags for every character (for isXXX() function series). */
    unsigned short  ausfType[256];
} __libc_GLocaleCtypeDefault;
__END_DECLS
#endif /* !__InnoTekLIBC_locale_h__ */


/** Bit masks for the ausfType member of __libc_GLocaleCtype
 *  and __libc_GLocaleCtypeDefault
 *
 * @remark These have identical values to the CT_* to speed up setlocale().
 * @{
 */
#define __UPPER     0x0001      /** Upper case alphabetic character. */
#define __LOWER     0x0002      /** Lower case alphabetic character. */
#define __DIGIT     0x0004      /** Digits 0-9. */
#define __SPACE     0x0008      /** White space and line ends. */
#define __PUNCT     0x0010      /** Punctuation marks. */
#define __CNTRL     0x0020      /** Control and format characters. */
#define __BLANK     0x0040      /** Space and tab. */
#define __XDIGIT    0x0080      /** Hex digits. */
#define __ALPHA     0x0100      /** Letters and linguistic marks. */
#define __ALNUM     0x0200      /** Alphanumeric. */
#define __GRAPH     0x0400      /** All except controls and space. */
#define __PRINT     0x0800      /** Everything except controls. */
#define __NUMBER    0x1000      /** Integral number. */
#define __SYMBOL    0x2000      /** Symbol. */
#define __ASCII     0x8000      /** In standard ASCII set. */
/** @} */


__BEGIN_DECLS
int     isalnum(int);
int     isalpha(int);
int     iscntrl(int);
int     isdigit(int);
int     isgraph(int);
int     islower(int);
int     isprint(int);
int     ispunct(int);
int     isspace(int);
int     isupper(int);
int     isxdigit(int);
int     toupper(int);
int     tolower(int);

#if __XSI_VISIBLE
int     _toupper(int);
int     _tolower(int);
int     isascii(int);
int     toascii(int);
#endif

#if __BSD_VISIBLE
/* @todo int	digittoint(int); */
int     isblank(int);
int	ishexnumber(int);
/* @todo int	isideogram(int); */
int	isnumber(int);
/* @todo int	isphonogram(int); */
/* @todo int	isrune(int); */
/* @todo int	isspecial(int); */
#endif
__END_DECLS

#define isalnum(c)  __istype((c), (__UPPER)|(__LOWER)|(__DIGIT))
#define isalpha(c)  __istype((c), (__UPPER)|(__LOWER))
#define iscntrl(c)  __istype((c), (__CNTRL))
#define isgraph(c)  __istype((c), (__PUNCT)|(__UPPER)|(__LOWER)|(__DIGIT))
#define islower(c)  __istype((c), (__LOWER))
#define isprint(c)  __istype((c), (__PRINT))
#define ispunct(c)  __istype((c), (__PUNCT))
#define isspace(c)  __istype((c), (__SPACE))
#define isupper(c)  __istype((c), (__UPPER))
#ifdef __UNIX_CHAR_CLASS__
/* BSD and some other UNIXes have non-standard definitions of these two at least. */
#define isdigit(c)  __isctype((c),(__DIGIT))
#define isxdigit(c) __isctype((c),(__XDIGIT))
#else
#define isdigit(c)  __istype((c),(__DIGIT))
#define isxdigit(c) __istype((c),(__XDIGIT))
#endif
#define tolower(c)  __tolower(c)
#define toupper(c)  __toupper(c)

#if __XSI_VISIBLE
#define _toupper(c) __toupper(c)
#define _tolower(c) __tolower(c)
#define	isascii(c)   (((c) & ~0x7F) == 0)
#define	toascii(c)   ((c) & 0x7F)
#endif

#if __BSD_VISIBLE
#define isblank(c)      __istype((c), (__BLANK))
#define ishexnumber(c)  __istype((c), (__XDIGIT))
#define isnumber(c)     __istype((c), (__DIGIT))
#endif


#if !defined(_DONT_USE_CTYPE_INLINE_) && \
    (defined(_USE_CTYPE_INLINE_) || defined(__GNUC__) || defined(__cplusplus))

__BEGIN_DECLS
static inline int __istype(int _c, unsigned _f)
{ return (__libc_GLocaleCtype.ausfType[_c & 0xff] & (_f)) != 0; }
static inline int __isctype(int _c, unsigned _f)
{ return (__libc_GLocaleCtypeDefault.ausfType[_c & 0xff] & (_f)) != 0; }
static inline int __toupper(int _c)
{ return __libc_GLocaleCtype.auchUpper[_c & 0xff]; }
static inline int __tolower(int _c)
{ return __libc_GLocaleCtype.auchLower[_c & 0xff]; }
__END_DECLS

#else

__BEGIN_DECLS
int __istype(int, unsigned);
int __isctype(int, unsigned);
int __toupper(int);
int __tolower(int);
__END_DECLS

#endif

#endif
